from pathlib import Path
from zipfile import ZIP_DEFLATED, ZipFile
from os import PathLike
import shutil

from typing import Union

def zip_dir(zip_name: str, source_dir: Union[str, PathLike]):
    src_path = Path(source_dir).expanduser().resolve(strict=True)
    with ZipFile(zip_name, 'w', ZIP_DEFLATED) as zf:
        for file in src_path.rglob('*'):
            zf.write(file, file.relative_to(src_path))


import os

from notion_client.helpers import get_id

from notion2md.console.formatter import *
from notion2md.exceptions import UnInitializedConfigException

def singleton(cls):
    instance = {}
    def get_instance(**kargs):
        if cls not in instance:
            if kargs:
                instance[cls] = cls(**kargs)
            else:
                raise UnInitializedConfigException
        return instance[cls]
    return get_instance

@singleton
class Config(object):
    __slots__ = ("file_name", "target_id", "output_path","tmp_path","download","unzipped","path_name")
    def __init__(self,**kargs):
        if "url" in kargs and kargs['url']:
            self.target_id = get_id(kargs['url'])
        elif "id" in kargs and kargs['id']:
            self.target_id = kargs['id']
        else:
            self.target_id = ""
        
        if "name" in kargs and kargs['name']:
            self.file_name = kargs['name']
        else:
            self.file_name = self.target_id

        if "path" in kargs and kargs['path']:
            self.path_name = kargs['path']
            self.output_path = os.path.abspath(kargs['path'])
            
        else:
            self.path_name = 'notion2md-output'
            self.output_path = os.path.join(os.getcwd(),'notion2md-output')

        if "download" in kargs and kargs['download']:
            self.download = True
        else:
            self.download = False

        if "unzipped" in kargs and kargs["unzipped"]:
            self.unzipped = True
            self.tmp_path = self.output_path
        else:
            self.unzipped = False
            self.tmp_path = os.path.join(os.getcwd(),'tmp')


import urllib.request as request
from urllib.parse import urlparse
import os
import uuid

from notion2md.console.formatter import *
from notion2md.config import Config
from notion2md.exceptions import UnInitializedConfigException

from cleo.io.io import IO

# Since external file block cannot guarantees
# whether the file is downloadable or not,
# there is no external file downloader
def downloader(url:str,io:IO=None) -> str:
    try:
        cfg= Config()
    except UnInitializedConfigException as e:
        if io:
            io.write_line(error(e))
        else:
            print(e)
    file_name = os.path.basename(urlparse(url).path)
    if cfg.download:
        if file_name:
            name,ext = os.path.splitext(file_name)

            downloaded_file_name = str(uuid.uuid4())[:8]+ext 
            fullpath = os.path.join(cfg.tmp_path,downloaded_file_name)
            
            if io:
                io.write_line(status("Downloading",f"{file_name}"))
                request.urlretrieve(url,fullpath)
                io.write_line(status("Downloaded",f'successfully downloaded "{file_name}" -> "{downloaded_file_name}"'))
            else:
                request.urlretrieve(url,fullpath)
            return name,downloaded_file_name
        else:
            if io:
                io.write_line(error(f"invalid {url}"))
    else:
        return file_name,url
        



import os,sys
import time
import shutil

from typing import Union

from notion2md.console.ui.indicator import progress
from notion2md.console.formatter import *
from notion2md.config import Config
from notion2md.notion_api import get_children
from notion2md.convertor.block import BlockConvertor
from notion2md.exceptions import UnInitializedConfigException
from notion2md.util import zip_dir

from cleo.helpers import option
from cleo.commands.command import Command

class ExportBlockCommand(Command):
    name = "block"
    description = "Export a Notion block object to markdown."

    options = [
        option("url", "u", "The url of Notion block object.", flag=False),
        option("id", "i", "The id of Notion block object.", flag=False),
        option("name", "n", "The name of Notion block object", flag=False),
        option("path", "p", "The path to save exported markdown file.", flag=False),
        option("download", None, "Download files/images inside of the block object", flag=True),
        option("unzipped", None, "Download unzipped output files/images", flag=True),
    ]
    help = """The block command retrieves and exports Notion <highlight>block object</highlight> to markdownfile.

- By default, the <highlight>id</highlight> of the block will be the name of markdown file, but if you path a value after <code>--name/-n</code>, the value will be the name of the markdown file.
    
- It will save the exported zip file in the path(<highlight>"notion-output/"</highlight> or <highlight>specific path you entered</highlight>)

- By default, it won't save files/images in the block object, but if you pass <code>--download</code> option, it will download files/images in the path(<highlight>"notion-output/"</highlight> or <highlight>specific path you entered</highlight>)

- By default, it will make a zip file in the output path, but if you want unzipped files, pass <code>--unzipped</code> option.
"""
    def status(self,st,msg):
        self.line(status(st,msg))
    
    def success(self,st,msg):
        self.line(sccuess(st,msg))
    
    def error(self,msg):
        self.line_error(error(msg))
        sys.exit(1)

    def handle(self):
        try:
            config = Config(**self.io.input.options)
        except UnInitializedConfigException as e:
            self.error(e)
        if not config.target_id:
            self.error("Notion2Md requires either id or url.")
        exporter = BlockConvertor(self.io)
        #Directory Checking and Creating
        if not os.path.exists(config.tmp_path):
            os.makedirs(config.tmp_path)
        if not os.path.exists(config.output_path):
            os.makedirs(config.output_path)
        start_time = time.time()
        #Get actual blocks
        self.line("")
        with progress(
            self.io,
            status("Retrieving","Notion blocks..."),
            sccuess("Retrieved","Notion blocks...")
        ):
            blocks = get_children(config.target_id)
        #Write(Export) Markdown file
        self.sccuess("Converting",f"<info>{str(len(blocks))}</info> blocks...")
        with open(os.path.join(config.tmp_path,config.file_name + '.md'),'w',encoding="utf-8") as output:
            output.write(exporter.convert(blocks))
        self.sccuess(
            "Converted",
            f"<info>{str(len(blocks))}</info> blocks to Markdown{f' <dim>({time.time() - start_time:0.1f}s)</dim>' if not self.io.is_debug() else ''}"
        )

        #Compress Output files into a zip file
        if not config.unzipped:
            zip_dir(os.path.join(config.output_path,config.file_name)+".zip",config.tmp_path)
            shutil.rmtree(config.tmp_path)
            extension = ".zip"
        else:
            extension = ".md"
        #Result and Time Check
        self.sccuess("Exported", f'"<info>{config.file_name}{extension}</info>" in "<info>./{config.path_name}/</info>"')
        self.line("")


from notion2md.convertor.block import BlockConvertor
from notion2md.config import Config
from notion2md.notion_api import get_children
from notion2md.util import zip_dir

import os
import shutil

def block_markdown_exporter(**kargs):
    config = Config(**kargs)
    #Directory Checking and Creating
    if not os.path.exists(config.tmp_path):
            os.makedirs(config.tmp_path)
    if not os.path.exists(config.output_path):
        os.mkdir(config.output_path)
    #Get actual blocks
    blocks = get_children(config.target_id)
    #Write(Export) Markdown file
    with open(os.path.join(config.output_path,config.file_name + '.md'),'w',encoding="utf-8") as output:
        output.write(BlockConvertor().convert(blocks))
    #Make Zip file and Delete tmp
    if not config.unzipped:
            zip_dir(os.path.join(config.output_path,config.file_name)+".zip",config.tmp_path)
            shutil.rmtree(config.tmp_path)
    
def block_string_exporter(**kargs):
    config = Config(**kargs)
    if (config.download and not os.path.exists(config.output_path)):
        os.mkdir(config.output_path)
    if not config.unzipped and not os.path.exists(config.tmp_path):
        os.makedirs(config.tmp_path)
    blocks = get_children(config.target_id)
    md = BlockConvertor().to_string(blocks)
    #Make Zip file and Delete tmp
    if not config.unzipped:
            zip_dir(os.path.join(config.output_path,config.file_name)+".zip",config.tmp_path)
            shutil.rmtree(config.tmp_path)
    return md


from notion2md import __version__

from cleo.application import Application as BaseApplication
from cleo.io.inputs.input import Input
from cleo.io.io import IO
from cleo.io.outputs.output import Output
from cleo.formatters.style import Style
from cleo.io.inputs.argument import Argument
from cleo.io.inputs.definition import Definition
from cleo.io.inputs.option import Option
from cleo.commands.command import Command
from cleo.commands.completions_command import CompletionsCommand
from cleo.commands.help_command import HelpCommand


from .commands.export_block import ExportBlockCommand

from typing import Optional,List

class Application(BaseApplication):
    def __init__(self):
        super(Application,self).__init__(
            "notion2md",__version__
        )
        self._default_command = 'block'
        self._single_command = True

    @property
    def default_commands(self) -> List[Command]:
        return [HelpCommand(),ExportBlockCommand(),CompletionsCommand()]
    
    def create_io(
        self, 
        input: Optional[Input] = None, 
        output: Optional[Output] = None, 
        error_output: Optional[Output] = None
        ) -> IO:
        io = super().create_io(input, output, error_output)

        formatter = io.output.formatter
        formatter.set_style("status",Style("green",options=['bold','dark']))
        formatter.set_style("sccuess",Style("green",options=['bold']))
        formatter.set_style("error",Style("red",options=['bold']))
        formatter.set_style("code",Style("red",options=['italic']))
        formatter.set_style("highlight",Style("blue",options=['bold']))
        formatter.set_style("dim",Style('default',options=['dark']))

        io.output.set_formatter(formatter)
        io.error_output.set_formatter(formatter)

        return io

    @property
    def _default_definition(self) -> Definition:
        return Definition(
            [
                Argument(
                    "command",
                    required=True,
                    description="The command to execute.",
                ),
                Option(
                    "--help",
                    "-h",
                    flag=True,
                    description=(
                        "Display help for the given command. "
                        f"When no command is given display help for the <info>{self._default_command}</info> command."
                    ),
                ),
                Option(
                    "--version",
                    "-V",
                    flag=True,
                    description="Display this application version.",
                ),
            ]
        )

def main():
    Application().run()

if __name__ == "__main__":
    main()


def error(msg):
    return f"\n<error>{'ERRER'+' ':>12}</error>{msg}\n"

def status(status,msg):
    return f"<status>{status+' ':>12}</status>{msg}"

def sccuess(status,msg):
    return f"<sccuess>{status+' ':>12}</sccuess>{msg}"
